#include <iostream>
#include <iomanip>
#include <vector>
#include <set>

struct Node 
{
    int id;
    double x, y ,z;
    bool vertex;

    Node() = default;
    Node(int, double, double, double, bool);

    bool operator==(const Node&) const;
    bool operator<(const Node&) const;
    friend std::ostream& operator<<(std::ostream&, const Node&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<Node>&);
};

struct FiniteElement 
{
    int element_id;
    int material_id;
    std::vector<int> nodes_id;

    friend std::ostream& operator<<(std::ostream&, const FiniteElement&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<FiniteElement>&);
};

struct BoundaryFiniteElement 
{
    int element_id;
    int border_id;
	std::vector<int> nodes_id;

    friend std::ostream& operator<<(std::ostream&, const BoundaryFiniteElement&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<BoundaryFiniteElement>&);
};

struct Edge 
{
	int head_id;
	int last_id;
	int self_id;
	Edge() = default;
	Edge(int, int);

	bool operator == (const Edge&) const;
};


class EdgeHash 
{
  public:
      std::size_t operator() (const Edge& _p) const
      {
          std::size_t o_seed = 0;
          if (_p.head_id > _p.last_id)
          {
              o_seed ^= std::hash<int>() (_p.head_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
              o_seed ^= std::hash<int>() (_p.last_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
          }
          else
          {
              o_seed ^= std::hash<int>() (_p.last_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
              o_seed ^= std::hash<int>() (_p.head_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
          }

          return o_seed;
      };
};







